@@ -14,10 +14,11 @@ import threading
 from contextlib import suppress
 from distutils.util import strtobool
 from pathlib import Path
-from typing import Any, Optional
+from typing import Any, Optional, Tuple
 
 import zmq
 import zmq.asyncio
+from executors import constants, global_settings
 from executors.connectors import connectors
 from executors.connectors.baseconnector import BaseStorageConnector
 from executors.connectors.hasher import StreamHasher
@@ -35,37 +36,52 @@ from executors.transfer import transfer_data
 from executors.zeromq_utils import ZMQCommunicator
 
 # Socket used to connect with the processing container.
-SOCKETS_PATH = Path(os.getenv("SOCKETS_VOLUME", "/sockets"))
-PROCESSING_SOCKET = SOCKETS_PATH / os.getenv(
-    "COMMUNICATION_PROCESSING_SOCKET", "_socket1.s"
-)
-PROCESSING_CONTAINER_TIMEOUT = int(os.getenv("CONTAINER_TIMEOUT", 300))
-
-UPLOAD_FILE_SOCKET = SOCKETS_PATH / os.getenv("UPLOAD_FILE_SOCKET", "_upload_socket.s")
+PROCESSING_SOCKET = constants.SOCKETS_VOLUME / constants.COMMUNICATION_PROCESSING_SOCKET
+UPLOAD_FILE_SOCKET = constants.SOCKETS_VOLUME / constants.UPLOAD_FILE_SOCKET
 
 # Listener IP and port are read from environment.
-LISTENER_IP = os.getenv("LISTENER_IP", "127.0.0.1")
-LISTENER_PORT = os.getenv("LISTENER_PORT", "53893")
+LISTENER_IP = os.getenv("LISTENER_SERVICE_HOST", "127.0.0.1")
+LISTENER_PORT = os.getenv("LISTENER_SERVICE_PORT", "53893")
 LISTENER_PROTOCOL = os.getenv("LISTENER_PROTOCOL", "tcp")
 DATA_ID = int(os.getenv("DATA_ID", "-1"))
-LOCATION_SUBPATH = Path(os.getenv("LOCATION_SUBPATH"))
 KEEP_DATA = bool(strtobool(os.environ.get("FLOW_MANAGER_KEEP_DATA", "False")))
 RUNNING_IN_KUBERNETES = bool(
     strtobool(os.environ.get("RUNNING_IN_KUBERNETES", "False"))
 )
-UPLOAD_CONNECTOR_NAME = os.getenv("UPLOAD_CONNECTOR_NAME", "local")
-
-DATA_VOLUME = Path(os.environ.get("DATA_VOLUME", "/data"))
 
 # How many file descriptors to receive over socket in a single message.
 DESCRIPTOR_CHUNK_SIZE = int(os.environ.get("DESCRIPTOR_CHUNK_SIZE", 100))
 
+MOUNTED_CONNECTORS = [
+    name for name in os.environ["MOUNTED_CONNECTORS"].split(",") if name
+]
+
+# Mapping between storage and connectors for this storage.
+# The values are tuples: (default connector, default mounted connector)
+STORAGE_CONNECTOR: dict[
+    str, Tuple[BaseStorageConnector, Optional[BaseStorageConnector]]
+] = {}
+
+# Configure container logger. All logs are output to stdout for further
+# processing.
+# The log level defaults to debug except for boto and google loggers, which
+# is set to warning due to them being extremely verbose in the debug mode.
+LOG_LEVEL = int(os.getenv("LOG_LEVEL", logging.DEBUG))
+BOTO_LOG_LEVEL = int(os.getenv("BOTO_LOG_LEVEL", logging.WARNING))
+GOOGLE_LOG_LEVEL = int(os.getenv("GOOGLE_LOG_LEVEL", logging.WARNING))
 
 logging.basicConfig(
     stream=sys.stdout,
-    level=logging.DEBUG,
+    level=LOG_LEVEL,
     format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
 )
+
+for boto_logger_name in ["botocore", "boto3", "s3transfer", "urllib3"]:
+    logging.getLogger(boto_logger_name).setLevel(BOTO_LOG_LEVEL)
+
+for google_logger in ["google"]:
+    logging.getLogger(google_logger).setLevel(GOOGLE_LOG_LEVEL)
+
 logger = logging.getLogger(__name__)
 
 logger.info("Starting communication container for data with id %d.", DATA_ID)
@@ -84,7 +100,7 @@ def purge_secrets():
             shutil.rmtree(path)
 
     try:
-        for root, dirs, files in os.walk(os.environ.get("SECRETS_DIR", "/secrets")):
+        for root, dirs, files in os.walk(constants.SECRETS_VOLUME):
             for f in files:
                 os.chmod(os.path.join(root, f), 0o700)
                 os.unlink(os.path.join(root, f))
@@ -163,8 +179,8 @@ class FakeConnector(BaseStorageConnector):
         """Get stream for data at the given URL."""
         # Skip 'subpath/' in "subpath/...".
         path = Path(url)
-        if LOCATION_SUBPATH < path:
-            url = path.relative_to(LOCATION_SUBPATH)
+        if global_settings.LOCATION_SUBPATH < path:
+            url = path.relative_to(global_settings.LOCATION_SUBPATH)
         file_stream = self.file_streams[os.fspath(url)]
         return file_stream
 
@@ -173,43 +189,50 @@ class Uploader(threading.Thread):
     """Upload referenced files to remote location."""
 
     def __init__(self, manager: "Manager", loop: asyncio.AbstractEventLoop):
-        """Initialization."""
+        """Initialize."""
         super().__init__()
         self._terminating = False
-        self.to_connector = connectors[UPLOAD_CONNECTOR_NAME].duplicate()
         self.manager = manager
         self.loop = loop
         self.ready = threading.Event()
 
-    def receive_file_descriptors(self, sock: socket.SocketType) -> dict[str, Any]:
+    def receive_file_descriptors(
+        self, sock: socket.SocketType
+    ) -> Tuple[str, dict[str, Any], bool]:
         """Receive file descriptors.
 
         See https://docs.python.org/3/library/socket.html#socket.socket.recvmsg .
 
         Protocol:
         1. Size of the filenames array (json) (64bites, #filenames_length).
-        2. Filenames & file descriptors.
-        3. Wait for single byte to confirm.
+        2. Connector name, filenames, presigned flag & file descriptors.
+        3. Send response in the form {"success": bool, "presigned_urls": list}.
         """
         filenames_length = int.from_bytes(sock.recv(8), byteorder="big")
         logger.debug(
             "Received file descriptors message of length: %d.", filenames_length
         )
         if filenames_length == 0:
-            return dict()
+            return ("", dict(), False)
         fds = array.array("i")  # Array of ints
         msg, ancdata, flags, addr = sock.recvmsg(
             filenames_length, socket.CMSG_LEN(DESCRIPTOR_CHUNK_SIZE * fds.itemsize)
         )
         logger.debug("Received file descriptors: %s, %s.", msg, ancdata)
-        filenames = json.loads(msg.decode())
+        storage_name, filenames, need_presigned_urls = json.loads(msg.decode())
         for cmsg_level, cmsg_type, cmsg_data in ancdata:
             if cmsg_level == socket.SOL_SOCKET and cmsg_type == socket.SCM_RIGHTS:
                 # Append data, ignoring any truncated integers at the end.
                 fds.frombytes(
                     cmsg_data[: len(cmsg_data) - (len(cmsg_data) % fds.itemsize)]
                 )
-        return dict(zip(filenames, fds))
+        return (storage_name, dict(zip(filenames, fds)), need_presigned_urls)
+
+    def send_message(self, sock, response):
+        """Send message to the socket."""
+        payload = json.dumps(response).encode()
+        sock.sendall(len(payload).to_bytes(8, byteorder="big"))
+        sock.sendall(payload)
 
     def run(self):
         """Start listening for file descriptors.
@@ -217,11 +240,11 @@ class Uploader(threading.Thread):
         Socket must be constructed here since it is not thread safe.
         """
         server_socket = socket.socket(family=socket.AF_UNIX)
-        server_socket.settimeout(PROCESSING_CONTAINER_TIMEOUT)
+        server_socket.settimeout(constants.CONTAINER_TIMEOUT)
         server_socket.bind(os.fspath(UPLOAD_FILE_SOCKET))
         server_socket.listen()
         self.ready.set()
-        # Wait for the connection up to PROCESSING_CONTAINER_TIMEOUT seconds.
+        # Wait for the connection up to CONTAINER_TIMEOUT seconds.
         # If it fails, the socket will be closed and process terminated if
         # it will try to save some files as outputs.
         try:
@@ -238,7 +261,11 @@ class Uploader(threading.Thread):
         with client:
             while not self._terminating:
                 try:
-                    file_descriptors = self.receive_file_descriptors(client)
+                    (
+                        storage_name,
+                        file_descriptors,
+                        need_presigned_urls,
+                    ) = self.receive_file_descriptors(client)
                     # Stop if the socket has been closed.
                     if not file_descriptors:
                         break
@@ -251,6 +278,7 @@ class Uploader(threading.Thread):
                     )
                     break
                 try:
+                    presigned_urls = []
                     to_transfer = []
                     logger.debug("Got %s", file_descriptors)
                     referenced_files: dict[str, dict[str, str]] = dict()
@@ -260,6 +288,8 @@ class Uploader(threading.Thread):
                         for file_name, file_descriptor in file_descriptors.items()
                     }
 
+                    # Get default connector for the given storage name.
+                    to_connector = STORAGE_CONNECTOR[storage_name][0]  # .duplicate()
                     for file_name in file_descriptors:
                         file_descriptor = file_descriptors[file_name]
                         stream = file_streams[file_name]
@@ -285,27 +315,47 @@ class Uploader(threading.Thread):
                         referenced_files[file_name]["chunk_size"] = chunk_size
                         referenced_files[file_name]["path"] = file_name
                         referenced_files[file_name]["size"] = file_size
+                        if need_presigned_urls:
+                            presigned_urls.append(
+                                to_connector.presigned_url(
+                                    global_settings.LOCATION_SUBPATH / file_name,
+                                    expiration=7 * 24 * 60 * 60,
+                                )
+                            )
                         stream.seek(0)
 
                     to_transfer = list(referenced_files.values())
                     from_connector = FakeConnector(
                         {"path": ""}, "File descriptors connector", file_streams
                     )
-                    transfer = Transfer(from_connector, self.to_connector)
-                    transfer.transfer_objects(LOCATION_SUBPATH, to_transfer)
+                    transfer = Transfer(from_connector, to_connector)
+                    transfer.transfer_objects(
+                        global_settings.LOCATION_SUBPATH, to_transfer
+                    )
                 except:
                     logger.exception("Exception uploading data.")
-                    client.sendall(b"0")
+                    self.send_message(client, {"success": False})
                     break
                 else:
-                    return_value = b"1"
                     if to_transfer:
                         future = asyncio.run_coroutine_threadsafe(
                             self.manager.send_referenced_files(to_transfer), self.loop
                         )
-                        if future.result().response_status == ResponseStatus.ERROR:
-                            return_value = b"0"
-                    client.sendall(return_value)
+                        future_response = future.result()
+                        # When data object is in state ERROR all responses will
+                        # have error status to indicate processing should
+                        # finish ASAP.
+                        if (
+                            future_response.response_status == ResponseStatus.ERROR
+                            and future_response.message_data != "OK"
+                        ):
+                            response = {"success": False}
+                        else:
+                            response = {
+                                "success": True,
+                                "presigned_urls": presigned_urls,
+                            }
+                    self.send_message(client, response)
                 finally:
                     for stream in file_streams.values():
                         stream.close()
@@ -321,7 +371,7 @@ class ListenerProtocol(BaseProtocol):
     def __init__(
         self, communicator: BaseCommunicator, processing_communicator: BaseCommunicator
     ):
-        """Initialization."""
+        """Initialize."""
         super().__init__(communicator, logger)
         self.processing_communicator = processing_communicator
 
@@ -358,7 +408,7 @@ class ProcessingProtocol(BaseProtocol):
     def __init__(
         self, communicator: BaseCommunicator, listener_communicator: BaseCommunicator
     ):
-        """Initialization."""
+        """Initialize."""
         super().__init__(communicator, logger)
         self.listener_communicator = listener_communicator
 
@@ -375,14 +425,14 @@ class ProcessingProtocol(BaseProtocol):
 
         This is needed in case empty dirs are referenced.
         """
+        subpath = global_settings.LOCATION_SUBPATH
         directories = message.message_data
         referenced_dirs = []
         for directory in directories:
-            # Make sure all referenced directories exists if
-            # UPLOAD_CONNECTOR_NAME equals "local".
-            if UPLOAD_CONNECTOR_NAME == "local":
-                destination_dir = DATA_VOLUME / directory
-                destination_dir.mkdir(parents=True, exist_ok=True)
+            if storage_connectors := STORAGE_CONNECTOR.get("data"):
+                if mounted_connector := storage_connectors[1]:
+                    destination_dir = mounted_connector.path / subpath / directory
+                    destination_dir.mkdir(parents=True, exist_ok=True)
             referenced_dirs.append({"path": os.path.join(directory, ""), "size": 0})
 
         return await self.listener_communicator.send_command(
@@ -417,7 +467,7 @@ class Manager:
     """
 
     def __init__(self):
-        """Initialization."""
+        """Initialize."""
         self.processing_communicator: Optional[BaseCommunicator] = None
         self.listener_communicator: Optional[BaseCommunicator] = None
         self.processing_container_connected = asyncio.Event()
@@ -515,7 +565,7 @@ class Manager:
                 logger.debug("Waiting for the processing container to connect")
                 await asyncio.wait_for(
                     self.processing_container_connected.wait(),
-                    PROCESSING_CONTAINER_TIMEOUT,
+                    constants.CONTAINER_TIMEOUT,
                 )
             except asyncio.TimeoutError:
                 message = "Unable to connect to the processing container."
@@ -539,6 +589,17 @@ class Manager:
                 # Start listening for messages from the communication and the
                 # processing container.
                 listener_task = asyncio.ensure_future(listener.communicate())
+
+                # Initialize settings constants by bootstraping.
+                response = await self.listener_communicator.send_command(
+                    Message.command("bootstrap", (DATA_ID, "communication"))
+                )
+                global_settings.initialize_constants(DATA_ID, response.message_data)
+                modify_connector_settings()
+                # Recreate connectors with received settings.
+                connectors.recreate_connectors()
+                set_default_storage_connectors()
+
                 processing_task = asyncio.ensure_future(processing.communicate())
                 listener_task.add_done_callback(self._communicator_stopped)
                 processing_task.add_done_callback(self._communicator_stopped)
@@ -576,10 +637,8 @@ class Manager:
             logger.debug("Terminating upload thread.")
             upload_thread.terminate()
             upload_thread.join()
-            # Secrets are read-only in kubernetes.
-            if not RUNNING_IN_KUBERNETES:
-                if not KEEP_DATA:
-                    purge_secrets()
+            if not KEEP_DATA:
+                purge_secrets()
 
             # Notify listener that the processing is finished.
             try:
@@ -600,6 +659,44 @@ class Manager:
             return return_code
 
 
+def set_default_storage_connectors():
+    """Set default mounted connector for each known storage."""
+    storages = global_settings.SETTINGS["FLOW_STORAGE"]
+    for storage_name, storage_settings in storages.items():
+        storage_connectors = connectors.for_storage(storage_name)
+        default_connector = storage_connectors[0]
+        default_mounted_connector = None
+        for connector in storage_connectors:
+            if connector.name in MOUNTED_CONNECTORS:
+                default_mounted_connector = connector
+                break
+
+        STORAGE_CONNECTOR[storage_name] = (default_connector, default_mounted_connector)
+
+
+def modify_connector_settings():
+    """Modify mountpoints and add processing and input connectors.
+
+    The path settings on filesystem connectors point to the path on the worker
+    node. They have to be remaped to a path inside container and processing and
+    input connector settings must be added.
+    """
+    connector_settings = global_settings.SETTINGS["STORAGE_CONNECTORS"]
+    storages = global_settings.SETTINGS["FLOW_STORAGE"]
+    connector_storage = {
+        connector_name: storage_name
+        for storage_name, storage_settings in storages.items()
+        for connector_name in storage_settings["connectors"]
+    }
+
+    # Point connector path to the correct mountpoint.
+    for connector_name in MOUNTED_CONNECTORS:
+        storage_name = connector_storage[connector_name]
+        connector_settings[connector_name]["config"]["path"] = Path(
+            f"/{storage_name}_{connector_name}"
+        )
+
+
 def sig_term_handler(manager_task: asyncio.Task):
     """Gracefully terminate the running process."""
     logger.debug("SIG_INT received, shutting down.")
