@@ -8,14 +8,16 @@ Manager Channels consumer.
 
 """
 import asyncio
+import logging
 
-import async_timeout
 from channels.consumer import AsyncConsumer
 from channels.layers import get_channel_layer
 from channels.testing import ApplicationCommunicator
 
 from . import state
 
+logger = logging.getLogger(__name__)
+
 
 async def send_event(message):
     """Construct a Channels event packet with the given message.
@@ -29,47 +31,50 @@ async def send_event(message):
     await get_channel_layer().send(state.MANAGER_CONTROL_CHANNEL, packet)
 
 
-async def run_consumer(timeout=None, dry_run=False):
+async def run_consumer(timeout=None):
     """Run the consumer until it finishes processing.
 
     :param timeout: Set maximum execution time before cancellation, or
         ``None`` (default) for unlimited.
-    :param dry_run: If ``True``, don't actually dispatch messages, just
-        dequeue them. Defaults to ``False``.
     """
-    channel = state.MANAGER_CONTROL_CHANNEL
-    scope = {
+    manager_channel = state.MANAGER_CONTROL_CHANNEL
+    manager_scope = {
         "type": "control_event",
-        "channel": channel,
+        "channel": manager_channel,
     }
+    manager_app = ApplicationCommunicator(ManagerConsumer(), manager_scope)
 
-    app = ApplicationCommunicator(ManagerConsumer, scope)
+    listener_channel = state.LISTENER_CONTROL_CHANNEL
+    listener_scope = {
+        "channel": listener_channel,
+    }
+    listener_app = ApplicationCommunicator(ListenerConsumer(), listener_scope)
 
     channel_layer = get_channel_layer()
 
-    async def _consume_loop():
+    async def _consume_loop(channel, scope, app):
         """Run a loop to consume messages off the channels layer."""
-        while True:
-            message = await channel_layer.receive(channel)
-            if dry_run:
-                continue
-            if message.get("type", {}) == "_resolwe_manager_quit":
-                break
+        message = await channel_layer.receive(channel)
+        while message.get("type", {}) != "_resolwe_manager_quit":
             message.update(scope)
             await app.send_input(message)
+            message = await channel_layer.receive(channel)
 
-    if timeout is None:
-        await _consume_loop()
-    try:
-        # A further grace period to catch late messages.
-        async with async_timeout.timeout(timeout or 1):
-            await _consume_loop()
-    except asyncio.TimeoutError:
-        pass
+    awaitables = [
+        asyncio.ensure_future(
+            _consume_loop(manager_channel, manager_scope, manager_app)
+        ),
+        asyncio.ensure_future(
+            _consume_loop(listener_channel, listener_scope, listener_app)
+        ),
+    ]
 
-    await app.wait()
-    # Shouldn't flush channels here in case there are more processes
-    # using the same Redis, since flushing is global.
+    await asyncio.wait(awaitables, timeout=timeout)
+    for awaitable in awaitables:
+        awaitable.cancel()
+
+    await manager_app.wait()
+    await listener_app.wait()
 
 
 async def exit_consumer():
@@ -78,6 +83,7 @@ async def exit_consumer():
         "type": "_resolwe_manager_quit",
     }
     await get_channel_layer().send(state.MANAGER_CONTROL_CHANNEL, packet)
+    await get_channel_layer().send(state.LISTENER_CONTROL_CHANNEL, packet)
 
 
 class ManagerConsumer(AsyncConsumer):
@@ -89,8 +95,26 @@ class ManagerConsumer(AsyncConsumer):
         from . import manager
 
         self.manager = manager
-        super().__init__(*args, **kwargs)
 
     async def control_event(self, message):
         """Forward control events to the manager dispatcher."""
-        await self.manager.handle_control_event(message["content"])
+        logger.debug("control_event got message %s", message)
+        try:
+            await self.manager.handle_control_event(message["content"])
+        except:
+            logger.exception("control_event exception.")
+
+
+class ListenerConsumer(AsyncConsumer):
+    """Channels consumer for handling manager events."""
+
+    def __init__(self, *args, **kwargs):
+        """Initialize a consumer instance with the given manager."""
+        # This import is local in order to avoid startup import loops.
+        from . import listener
+
+        self.listener = listener
+
+    async def terminate_worker_event(self, message):
+        """Forward control events to the manager dispatcher."""
+        await self.listener.terminate_worker(message["identity"])
