@@ -1,6 +1,4 @@
 # pylint: disable=missing-docstring,too-many-lines
-import io
-import json
 import os
 import shutil
 from datetime import timedelta
@@ -27,12 +25,7 @@ from resolwe.flow.models.data import Data, DataDependency, hydrate_size, render_
 from resolwe.flow.models.utils import hydrate_input_references, referenced_files
 from resolwe.flow.views import DataViewSet
 from resolwe.test import TestCase, TransactionTestCase
-from resolwe.test.utils import create_data_location
-
-try:
-    import builtins  # py3
-except ImportError:
-    import __builtin__ as builtins  # py2
+from resolwe.test.utils import create_data_location, save_storage
 
 
 class DataModelNameTest(TransactionTestCase):
@@ -49,7 +42,7 @@ class DataModelNameTest(TransactionTestCase):
                     "required": False,
                 }
             ],
-            run={"language": "bash", "program": 'echo {"stat": "42"}'},
+            run={"language": "bash", "program": "echo test"},
         )
 
         data = Data.objects.create(contributor=self.contributor, process=process)
@@ -86,6 +79,7 @@ class DataModelNameTest(TransactionTestCase):
                     "required": False,
                 }
             ],
+            run={"language": "bash", "program": "echo test"},
         )
 
         with transaction.atomic():
@@ -153,6 +147,12 @@ class DataModelTest(TestCase):
         self.assertEqual(data.name[-3:], "...")
 
     def test_hydrate_file_size(self):
+        """Hydrate file size.
+
+        NOTE: It is no longer called every time the data object is saved os it
+        needs to be called explicitly. The sizes are calculated and  reported
+        by the communication container when object processing is done.
+        """
         proc = Process.objects.create(
             name="Test process",
             contributor=self.contributor,
@@ -174,14 +174,13 @@ class DataModelTest(TestCase):
         os.makedirs(dir_path)
 
         with self.assertRaises(ValidationError):
-            data.save()
+            hydrate_size(data)
 
         file_path = os.path.join(dir_path, "output.txt")
         with open(file_path, "w") as fn:
             fn.write("foo bar")
 
-        data.save()
-
+        hydrate_size(data)
         self.assertEqual(data.output["output_file"]["size"], 7)
 
     def test_dependencies_single(self):
@@ -641,7 +640,9 @@ class DuplicateTestCase(TestCase):
         data_location.data.add(data2)
         data2.output = {"json_field": {"foo": "bar"}}
         data2.status = Data.STATUS_DONE
+        save_storage(data2)
         data2.save()
+
         data2.migration_history.create(migration="migration_1")
         data2.migration_history.create(migration="migration_2")
 
@@ -745,7 +746,6 @@ class DuplicateTestCase(TestCase):
                 {"name": "json_field", "type": "basic:json:"},
             ],
         )
-
         input_data = Data.objects.create(contributor=self.user, process=process1)
 
         data = Data.objects.create(
@@ -755,6 +755,7 @@ class DuplicateTestCase(TestCase):
         data_location.data.add(data)
         data.output = {"json_field": {"foo": "bar"}}
         data.status = Data.STATUS_DONE
+        save_storage(data)
         data.save()
         data.migration_history.create(migration="migration_1")
 
@@ -1252,7 +1253,6 @@ class HydrateFileSizeUnitTest(TestCase):
 class StorageModelTestCase(TestCase):
     def setUp(self):
         super().setUp()
-
         self.proc = Process.objects.create(
             name="Test process",
             contributor=self.contributor,
@@ -1261,54 +1261,6 @@ class StorageModelTestCase(TestCase):
             ],
         )
 
-    def test_save_storage(self):
-        """`basic:json:` fields are stored in Storage"""
-        data = Data.objects.create(
-            name="Test data",
-            contributor=self.contributor,
-            process=self.proc,
-        )
-
-        data.output = {"json_field": {"foo": "bar"}}
-        data.save()
-        first_storage = data.output["json_field"]
-
-        data.output = {"json_field": {"foo": "bar"}}
-        data.status = Data.STATUS_DONE
-        data.save()
-
-        self.assertEqual(Storage.objects.count(), 1)
-        storage = Storage.objects.first()
-        self.assertEqual(data.output["json_field"], storage.pk)
-        # Make sure that the storage is updated and not replaced.
-        self.assertEqual(first_storage, storage.pk)
-
-    def test_save_storage_file(self):
-        """File is loaded and saved to storage"""
-        data = Data.objects.create(
-            name="Test data",
-            contributor=self.contributor,
-            process=self.proc,
-        )
-        data_location = create_data_location()
-        data_location.data.add(data)
-
-        data.output = {"json_field": "json.txt"}
-        data.status = Data.STATUS_DONE
-
-        json_file = io.StringIO(json.dumps({"foo": "bar"}))
-
-        isfile_mock = MagicMock(return_value=True)
-        open_mock = MagicMock(return_value=json_file)
-        with patch.object(os.path, "isfile", isfile_mock):
-            with patch.object(builtins, "open", open_mock):
-                data.save()
-
-        self.assertEqual(Storage.objects.count(), 1)
-        storage = Storage.objects.first()
-        self.assertEqual(data.output["json_field"], storage.pk)
-        self.assertEqual(storage.json, {"foo": "bar"})
-
     def test_delete_data(self):
         """Orphaned storages are deleted when data object is deleted"""
         storage_referenced_by_both = Storage.objects.create(
@@ -1344,9 +1296,9 @@ class StorageModelTestCase(TestCase):
             contributor=self.contributor,
             process=self.proc,
         )
-
         data.output = {"json_field": {"foo": {"moo": "bar"}}}
         data.status = Data.STATUS_DONE
+        save_storage(data)
         data.save()
 
         # Annotation with specific JSON field.
@@ -1424,18 +1376,6 @@ class UtilsTestCase(TestCase):
             status=Data.STATUS_ERROR,
             process=process,
             # Workaround for skipping the validation.
-            output={
-                "file": {"file": "some-file", "refs": ["ref1"]},
-                "file_list": [
-                    {"file": "some-file", "refs": ["ref2"]},
-                    {"file": "another-file"},
-                ],
-                "dir": {"dir": "some-dir", "refs": ["ref3"]},
-                "dir_list": [
-                    {"dir": "some-dir", "refs": ["ref4"]},
-                    {"dir": "another-dir"},
-                ],
-            },
             descriptor_schema=descriptor_schema,
             descriptor={
                 "annotation": "my-annotation",
@@ -1444,6 +1384,20 @@ class UtilsTestCase(TestCase):
         )
         data_location = create_data_location()
         data_location.data.add(data)
+        data.output = {
+            "file": {"file": "some-file", "refs": ["ref1"]},
+            "file_list": [
+                {"file": "some-file", "refs": ["ref2"]},
+                {"file": "another-file"},
+            ],
+            "dir": {"dir": "some-dir", "refs": ["ref3"]},
+            "dir_list": [
+                {"dir": "some-dir", "refs": ["ref4"]},
+                {"dir": "another-dir"},
+            ],
+        }
+        data.save()
+
         input_schema = [
             {
                 "name": "data",
@@ -1453,7 +1407,11 @@ class UtilsTestCase(TestCase):
         input_ = {"data": data.pk}
         hydrate_input_references(input_, input_schema)
 
-        path_prefix = data.location.get_path()
+        # TODO: why this worked?
+        # path_prefix = data.location.get_path()
+        from resolwe.flow.managers import manager
+
+        path_prefix = manager.get_executor().resolve_data_path(data=data)
 
         self.assertEqual(
             input_["data"]["__descriptor"], {"annotation": "my-annotation"}
@@ -1541,7 +1499,6 @@ class UtilsTestCase(TestCase):
         )
         refs = referenced_files(data_mock)
         refs.remove("jsonout.txt")
-        refs.remove("stderr.txt")
         refs.remove("stdout.txt")
         return refs
 
@@ -1710,7 +1667,6 @@ class UtilsTestCase(TestCase):
         )
         refs = referenced_files(data_mock)
         refs.remove("jsonout.txt")
-        refs.remove("stderr.txt")
         refs.remove("stdout.txt")
 
         refs = self._test_referenced_files(field_schema, output)
@@ -1741,6 +1697,5 @@ class UtilsTestCase(TestCase):
         )
         refs = referenced_files(data_mock)
         refs.remove("jsonout.txt")
-        refs.remove("stderr.txt")
         refs.remove("stdout.txt")
         self.assertSetEqual(set(refs), {"file1", "file2"})
