@@ -18,9 +18,6 @@ from .fields import (
 )
 
 DATA_ID = int(os.getenv("DATA_ID", "-1"))
-DATA_LOCAL_VOLUME = Path(os.environ.get("DATA_LOCAL_VOLUME", "/data_local"))
-DATA_VOLUME = Path(os.environ.get("DATA_VOLUME", "/data"))
-DATA_ALL_VOLUME = Path(os.environ.get("DATA_ALL_VOLUME", "/data_all"))
 
 
 class ModelField(Field):
@@ -76,10 +73,12 @@ FIELDS_MAP = {
     "SlugField": StringField,
 }
 
+_hydrate_cache: Dict[int, str] = dict()
+
 
 def hydrate_if_needed(value, model_instance, field_name, field):
     """Hydrate path if needed."""
-    # Only hidrate paths on output of data objects that we are not processing.
+    # Only hidrate paths on outputs of data objects that we are not processing.
     if (
         model_instance._model_name == "Data"
         and model_instance.id != DATA_ID
@@ -87,7 +86,11 @@ def hydrate_if_needed(value, model_instance, field_name, field):
         and field.get_field_type()
         in ("basic:file", "list:basic:file", "basic:dir", "list:basic:dir")
     ):
-        base_path = DATA_ALL_VOLUME / str(model_instance.location_id)
+        if model_instance.id not in _hydrate_cache:
+            _hydrate_cache[model_instance.id] = Path(
+                communicator.resolve_data_path(model_instance.id)
+            )
+        base_path = _hydrate_cache[model_instance.id] / str(model_instance.location_id)
         if field.get_field_type().startswith("basic"):
             value.path = os.fspath(base_path / value.path)
         else:
@@ -107,7 +110,7 @@ class JSONDescriptor(MutableMapping[str, Any]):
         read_only=False,
         cache: Optional[Dict[str, Any]] = None,
     ):
-        """Initialization."""
+        """Initialize."""
         self._model = model
         self._model_name = self._model._model_name
         self._app_name = self._model._app_name
@@ -283,7 +286,7 @@ class Model(metaclass=ModelMetaclass):
     _filter_response_fields = []
 
     def __init__(self, pk: int):
-        """Initialization."""
+        """Initialize."""
         self._pk = pk
         self._cache: Dict[str, Any] = {"id": pk}
 
@@ -338,8 +341,14 @@ class Model(metaclass=ModelMetaclass):
         pks = communicator.filter_objects(
             cls._app_name, cls._model_name, filters, ["id"]
         )
-        if len(pks) != 1:
-            raise RuntimeError("Exactly one object should match the given criteria.")
+        if len(pks) > 1:
+            raise RuntimeError(
+                f"Exactly one object should match the given criteria, received {len(pks)}."
+            )
+        elif len(pks) == 0:
+            raise RuntimeError(
+                "No objects match the given criteria or no permission to read object."
+            )
         return cls(pks[0][0])
 
     @classmethod
@@ -382,6 +391,8 @@ class Model(metaclass=ModelMetaclass):
 
         It is used by the fields (descriptors) to retrieve the data from the
         server. The cache is used if applicable.
+
+        :raises RuntimeError: when no data is received.
         """
         if field.name not in self._cache:
             result = communicator.get_model_fields(
@@ -389,8 +400,16 @@ class Model(metaclass=ModelMetaclass):
             )
             if len(result) > 1:
                 result = [e[field.name] for e in result]
-            else:
+            elif len(result) == 1:
                 result = result[field.name]
+            else:
+                raise RuntimeError(
+                    (
+                        f"No data received for property '{field.name}' of "
+                        f"model {self._model_name} with id {self._pk}. Check "
+                        "property name and permissions."
+                    )
+                )
             self._cache[field.name] = field.clean(result)
         return self._cache[field.name]
 
