@@ -19,6 +19,17 @@ class LocalFilesystemConnector(BaseStorageConnector):
         self.path = config["path"]
         self.supported_hash = ["crc32c", "md5", "awss3etag"]
         self.multipart_chunksize = self.CHUNK_SIZE
+        self.get_ensures_data_integrity = True
+        self.put_ensures_data_integrity = True
+
+    @property
+    def mountable(self):
+        """Can a connector be mounted inside container."""
+        return True
+
+    def prepare_url(self, url, **kwargs):
+        """Prepare all the necessary for a new location."""
+        (self.base_path / url).mkdir(mode=kwargs.get("dir_mode", 0o755), parents=True)
 
     @validate_urls
     @validate_url
@@ -48,6 +59,41 @@ class LocalFilesystemConnector(BaseStorageConnector):
                 f.write(data)
                 data_remaining = len(data) == chunk_size
 
+    def multipart_push_start(self, url, size=None):
+        """Start a multipart upload.
+
+        :returns: the upload id.
+        :raises AssertionError: when url exists or upload size is not given.
+        """
+        assert size is not None, f"{self}: size of multipart upload must be specified."
+        path = self.base_path / url
+        assert not path.exists(), f"Path {url} already exists, aborting upload."
+        if not path.is_file():
+            with path.open("wb") as stream:
+                stream.truncate(size)
+        return url
+
+    def multipart_push(self, upload_id, url, part_number, chunk_size, data, md5=None):
+        """Upload a single part of multipart upload."""
+        path = self.base_path / url
+        assert path.is_file(), f"{self}: multipart upload file {path} does not exist."
+        with path.open("r+b") as stream:
+            stream.seek((part_number - 1) * chunk_size)
+            shutil.copyfileobj(data, stream, 1024 * 1024)
+        return dict()
+
+    def multipart_push_complete(self, upload_id, url, completed_chunks):
+        """Complete the multipart push."""
+        path = self.base_path / url
+        assert path.is_file(), f"{self}: multipart upload file {path} does not exist."
+        return dict()
+
+    def multipart_push_abort(self, upload_id, url):
+        """Abort multiport upload."""
+        path = self.base_path / url
+        if path.is_file():
+            path.unlink()
+
     @validate_url
     def get(self, url, stream, chunk_size=BaseStorageConnector.CHUNK_SIZE):
         """Get data from the given URL and write it into the given stream."""
@@ -87,7 +133,7 @@ class LocalFilesystemConnector(BaseStorageConnector):
     @validate_url
     def get_hashes(self, url, hash_types):
         """Get the hash of the given type for the given object."""
-        hasher = StreamHasher(chunk_size=self.multipart_chunksize)
+        hasher = StreamHasher(chunk_size=self.multipart_chunksize, hashes=hash_types)
         path = self.base_path / url
         if not path.exists():
             return None
@@ -98,7 +144,7 @@ class LocalFilesystemConnector(BaseStorageConnector):
     @validate_url
     def get_hash(self, url, hash_type):
         """Get the hash of the given type for the given object."""
-        hasher = StreamHasher(chunk_size=self.multipart_chunksize)
+        hasher = StreamHasher(chunk_size=self.multipart_chunksize, hashes=[hash_type])
         path = self.base_path / url
         if not path.exists():
             return None
@@ -135,6 +181,6 @@ class LocalFilesystemConnector(BaseStorageConnector):
         :returns: URL that can be used to access object or None.
         """
         force_download = "?force_download=1" if force_download else ""
-        public_url = Path(self.config.get("public_url", "/local_data"))
+        public_url = Path(self.config.get("public_url", ""))
         resource_url = public_url / url
         return resource_url.as_posix() + force_download
