@@ -124,7 +124,7 @@ class Transfer:
 
         futures = paralelize(
             objects=objects_to_transfer,
-            worker=partial(self._transfer_chunk, url),
+            worker=partial(self.transfer_chunk, url),
             max_threads=max_threads,
         )
 
@@ -146,9 +146,13 @@ class Transfer:
 
         return None if objects_stored is objects else objects_stored
 
-    def _transfer_chunk(self, url: Path, objects: Iterable[dict]) -> bool:
+    def transfer_chunk(self, url: Path, objects: Iterable[dict]) -> bool:
         """Transfer a single chunk of objects.
 
+        When objects have properties `from_base_url` and `to_base_url` they
+        override the `url` argument.
+
+        :raises DataTransferError: on failure.
         :returns: True on success.
         """
         to_connector = self.to_connector.duplicate()
@@ -157,7 +161,12 @@ class Transfer:
             # Do not transfer directories.
             if not entry["path"].endswith("/"):
                 if not self.transfer(
-                    url, entry, url, Path(entry["path"]), from_connector, to_connector
+                    entry.get("from_base_url", url),
+                    entry,
+                    entry.get("to_base_url", url),
+                    Path(entry["path"]),
+                    from_connector,
+                    to_connector,
                 ):
                     raise DataTransferError()
         return True
@@ -206,6 +215,27 @@ class Transfer:
 
         from_url = Path(from_base_url) / object_["path"]
         hashes = {type_: object_[type_] for type_ in ["md5", "crc32c", "awss3etag"]}
+
+        skip_final_hash_check = (
+            from_connector.get_ensures_data_integrity
+            and to_connector.put_ensures_data_integrity
+        )
+        if skip_final_hash_check:
+            # When final check is skipped make sure that the input connector
+            # hash equals to the hash given by the _object (usually read from
+            # the database).
+            hash_to_check = next(
+                hash for hash in from_connector.supported_hash if hash in hashes.keys()
+            )
+            from_connector_hash = from_connector.get_hash(from_url, hash_to_check)
+            expected_hash = object_[hash_to_check]
+            if expected_hash != from_connector_hash:
+                raise DataTransferError(
+                    f"Connector {from_connector} has {from_connector_hash} stored  "
+                    f"as {from_connector_hash} hash for object "
+                    f"{from_url}, expected {expected_hash}."
+                )
+
         common_hash_type = next(
             e for e in to_connector.supported_hash if e in hashes.keys()
         )
@@ -240,8 +270,11 @@ class Transfer:
                 if future.exception() is not None:
                     executor.shutdown(wait=False)
 
+            # The constant 1 is added to the object size to make sure the
+            # buffer_size is at least 1. Otherwise uploading files of size 0
+            # will cause a deadlock.
             data_stream = CircularBuffer(
-                buffer_size=min(200 * 1024 * 1024, object_["size"])
+                buffer_size=min(200 * 1024 * 1024, object_["size"] + 1)
             )
             with concurrent.futures.ThreadPoolExecutor() as executor:
                 download_task = executor.submit(
@@ -278,10 +311,16 @@ class Transfer:
                 raise DataTransferError("\n\n".join(messages))
 
         # Check hash of the uploaded object.
-        if from_hash != to_connector.get_hash(to_base_url / to_url, common_hash_type):
-            with suppress(Exception):
-                to_connector.delete(to_base_url, [to_url])
-            raise DataTransferError()
+        if not skip_final_hash_check:
+            to_hash = to_connector.get_hash(to_base_url / to_url, common_hash_type)
+            if from_hash != to_hash:
+                with suppress(Exception):
+                    to_connector.delete(to_base_url, [to_url])
+                raise DataTransferError(
+                    f"Hash {common_hash_type} does not match while transfering "
+                    f"{from_url} -> {to_base_url/to_url}: using hash type "
+                    f"{common_hash_type}: expected {from_hash}, got {to_hash}."
+                )
 
         # Store computed hashes as metadata for later use.
         to_connector.set_hashes(to_base_url / to_url, hashes)
